/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.expression.api.BinaryOperator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;

public class Concat extends BasicBinaryOperator implements BinaryOperator {

	/**
	 * Returns the concatenation of two strings.
	 * @param left  the first Expression to be concatenated
	 * @param right  the second Expression to be concatenated
	 */
	public Concat(Expression left, Expression right) {
		this.left = left;
		this.right = right;
	}

	public Concat() {
	}

	public org.objectweb.medor.expression.api.Operand evaluate(ParameterOperand[] pos, Object o)
		throws ExpressionException {
		try {
			result.setValue(
					evaluate(
							left.evaluate(pos, o).getString(),
							right.evaluate(pos, o).getString()));
		}
		catch (NullPointerException e) {
			// result == null or missing children objects
			throw new IllegalStateException("Unevaluable Expression: Not compiled");
		}
		return result;
	}

	public String evaluate(String op1, String op2) {
		return op1.concat(op2);
	}

	public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
		if ((left != null) && (right != null)) {
			left.compileExpression();
			right.compileExpression();
			if (!(left.getType() == null || right.getType() == null)) {
				if ((left.getType().getTypeCode() == PType.TYPECODE_STRING) &&
					(right.getType().getTypeCode() == PType.TYPECODE_STRING)) {
					type = PTypeSpace.STRING;
					result = new BasicVariableOperand(type);
					verified = true;
				}
				else
				// it is not a string type
					throw new TypingException("Attempt arg type : String, String");
			}
			else
				throw new MalformedExpressionException("null children value");
		}
		else
		// Operands not yet assigned
			throw new MalformedExpressionException("null children value");
		return result;
	}

	public String getOperatorString() {
		return Operator.CONCAT;
	}
}
